<?php

namespace Novitax\Wintax\Controller\Index;

class Index extends \Magento\Framework\App\Action\Action
{
    
    protected $scopeConfig;
    protected $connection;
    protected $resource;
    protected $stockRegistry;
    protected $orderRepository;
    const NOVITAX_KEY = 'wintax/general/NOVITAX_KEY';

    public function __construct(
        \Magento\Framework\App\Action\Context $context,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository
    ) {
        $this->scopeConfig = $scopeConfig;
        
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $this->resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
        $this->connection = $this->resource->getConnection();

        $this->stockRegistry = $stockRegistry;
        $this->orderRepository = $orderRepository;

        parent::__construct($context);
    }


    public function execute()
    {
      
		if (isset($_POST['wintax_method'])) {

			$r = $this->check_key($_POST['wintax_key']);
			if (!$r) {
				header('HTTP/1.1 401 Unauthorized');
				exit;
			}

			switch ($_POST['wintax_method']) {
				case 'update_stock':
				  $this->update_stock();
				  exit;
				case 'get_invoice_data':
				  $this->get_invoice_data();
				  exit;
				case 'send_invoice':
				  $this->send_invoice();
				  exit;                     
				case 'test_connection':
				  $this->test_connection();
				  exit;         
				default:
				  header('HTTP/1.1 400 Bad Request');
				  exit;
			}
		  
		}
		else {
			return;
		}


    }

    // kulcs összevetése a tárolttal
	public function check_key($key) {
		
      	$storeScope = \Magento\Store\Model\ScopeInterface::SCOPE_STORE;
      	$novitax_key = $this->scopeConfig->getValue(self::NOVITAX_KEY, $storeScope);

		if (!empty($key) && ($novitax_key == $key)) {
			return true;
		}
		else {
			return false;
		}
	}


	// kapcsolat ellenőrzés
	public function test_connection() {
		echo 'OK';
	}


	// számlázandó rendelések xml-jeinek összefűzése és elküldése
	public function get_invoice_data() {

		$xml = '<?xml version="1.0" encoding="UTF-8"?><Orders>';
		$ids = array();

        $tableName = $this->resource->getTableName('novitax');

        $sql = "SELECT id_order, xml FROM $tableName WHERE status = 'new'";
        $results = $this->connection->fetchAll($sql); 

		if ($results) {
			foreach ($results as $result) {
				$xml .= $result['xml'];
				$ids[] = $result['id_order'];

				$this->orderRepository->get($result['id_order'])->addStatusHistoryComment('Számlázási adatok átadva Wintaxnak.')->save();

			}

			if (!empty($ids)) {
				$sql = "UPDATE $tableName SET status = 'sent' WHERE id_order IN ( " . implode(',', $ids) . " )";
          		$this->connection->query($sql);
			}

		}


		  $xml .= '</Orders>';

		  header("Content-type: text/xml; charset=utf-8");
		  echo $xml;  

	}	

	// számlázott rendelések számlaszámának (pdf számláinak) fogadása
	public function send_invoice() {

		$order_id = (int)$_POST['order_id'];
		$invoice_num = $_POST['invoice_num'];

		//$file = $_POST['invoice_pdf'];
		$tableName = $this->resource->getTableName('novitax');

		$sql = "UPDATE $tableName SET status = 'generated', invoice_num = :NT_INVOICE_ID WHERE id_order = :NT_ORDER_ID LIMIT 1";
		$binds = array('NT_INVOICE_ID' => $invoice_num, 'NT_ORDER_ID' => $order_id);
		$this->connection->query($sql, $binds);

		try {
			$this->orderRepository->get($order_id)->addStatusHistoryComment('Számlázás sikeresen megtörtént Wintaxban, számlaszám: ' . $invoice_num)->save();
			echo 'OK';
		}
		catch (\Magento\Framework\Exception\NoSuchEntityException $e) {
			header('HTTP/1.1 422 Unprocessable Entity');
			echo 'order_id not exists: ' . $order_id . "\n";
			exit;
		} 


	}


	// készlet adatok frissítése wintax felől
	public function update_stock() {

		libxml_use_internal_errors(true);
		$xml = simplexml_load_string(stripslashes($_POST['wintax_xml']));
		
		if ($xml === false) {
			header('HTTP/1.1 422 Unprocessable Entity');
			echo 'malformed xml';
			exit;
		}

		$found = 0;

		foreach ($xml->StockItem as $product) {

			$sku = $product->EAN;
			$price = (int)$product->Price;
			$q = (int)$product->Amount;

			if (!empty($sku)) {

				try {
				
					$stockItem = $this->stockRegistry->getStockItemBySku($sku);
			    	$stockItem->setQty($qty);
			    	$stockItem->setIsInStock((bool)$qty); 
			    	$this->stockRegistry->updateStockItemBySku($sku, $stockItem);
			    	$found++;
			    
			    } catch (\Magento\Framework\Exception\NoSuchEntityException $e) {
					// echo 'Not exists: ' . $sku . "\n";
				}


			}
		  
		}

		echo 'OK: ' . $found;
	}


}